import java.sql.*;

/**
 * This class is used to read and write the data concerning 
 * the properties in the DB.
 *<P>
 * It exists different kinds of properties. The property can
 * concern default settings, user's preferences, presentation
 * preferences or foil preferences.
 * @author Willy Picard.
 * @version 20.07.1998 1
 */
public class dbProperty
{
    /**
     * For debug purpose.
     */
    private boolean DEBUG = true;
    
    /**
     * A utility used to parse the String in order to replace the
     * character ' by ''.
     * @see
     */
    private TextFormator tf;
    
    
    
    /**
     * Constant used to indicate that this property is 
     * concerning the default settings.
     */
    public final static int DEFAULT = 0;

    /**
     * Constant used to indicate that this property is 
     * concerning the user's preferences.
     */
    public final static int USER = 1;

    /**
     * Constant used to indicate that this property is 
     * concerning the presentation preferences.
     */
    public final static int PRESENTATION = 2;
    
    /**
     * Constant used to indicate that this property is 
     * concerning the foil preferences.
     */
    public final static int FOIL = 3;
    
    /**
     * Constant used to indicate that this property is 
     * concerning the color settings.
     */
    public final static int COLOR = 0;

    /**
     * Constant used to indicate that this property is 
     * concerning the image settings.
     */
    public final static int IMAGE = 1;
    
    /**
     * Constant used to indicate that this property is 
     * concerning the font settings.
     */
    public final static int FONT = 2;
    
    /**
     * Constant used to indicate that this property is 
     * concerning the image size settings.
     */
    public final static int IMAGE_SIZE = 3;
    
    /**
     * Constant used to indicate that this property is 
     * concerning the configuration settings.
     */
    public final static int CONFIGURATION = 4;
    
    /**
     * Constant used to indicate that this property is 
     * concerning the others settings.
     */
    public final static int OVERALL = 5;

    /**
     * The identifier of the user, the presentation or the
     * foil for which the property is set.
     */
    private int id;
    
    /**
     * The name of the property.
     */
    private String name;
    
    /**
     * The value of the property.
     */
    private String value;
    
    /**
     * The property level (default, user or presentation).
     */
    int level;
    
    /**
     * The kind of property (image, color, font, image sizes, 
     * configuration, overall).
     */
    int type;
    
    /**
     * Constructor.
     * @param name the name of the property.
     * @param value the value of the property.
     * @param id the identifier of the user, the presentation or the
     * foil for which the property is set.
     * @param level the property level (general, user or presentation).
     * @param type the type of property (image, color, etc.)
     */
     public dbProperty(String name,
                       String value,
                       int id,
                       int level,
                       int type)
     {
        this.name = name;
        this.value = value;
        this.id = id;
        this.level = level;
        this.type = type;
     }
     
     /**
      * Writes the data to the DB.
      * @param connection the connection to the DB.
      */
     public void sendToDB(Connection connection)
     {
        if (isPropertySet(connection))
            updateToDB(connection);
        else
            insertToDB(connection);
     }
     
     /**
      * Checks if this property is set in the DB.
      * @param connection the connection to the DB.
      * @return true if this property is set in the DB.
      */
     public boolean isPropertySet(Connection connection)
     {
        boolean result = false;
        
        tf = new TextFormator();
        String identificatorName = "";
        String pridName = "";
        String sql = "SELECT * FROM ";

        switch (level)
        {
            case (DEFAULT) :
                identificatorName = "";
                break;
            case (USER) :
                identificatorName = " AND USER_UID="+id;
                sql += "USER_";
                break;
            default :
                identificatorName = ", AND PRES_PID="+id;
                sql += "PRES_";
                break;
        }

        switch (type)
        {
            case (COLOR) :
                sql += "COLOR_PROPERTY, COLOR_PROPERTY ";
                pridName = "COLOR_PROP_PRID";
                break;
            case (IMAGE) :
                sql += "IMAGE_PROPERTY, IMAGE_PROPERTY ";
                pridName = "IMAGE_PROP_PRID";
                break;
            default :
                sql += "GENERAL_PROPERTY, GENERAL_PROPERTY ";
                pridName = "GEN_PROP_PRID";
                break;
        }
        
        sql += "WHERE SEARCH_NAME='"+tf.prime(name)+"'";
        sql += identificatorName;
        if (type != DEFAULT)
            sql += " AND PRID="+pridName;
        
        if (DEBUG)
            System.out.println(sql);
      
        try
        {
            Statement stmt = connection.createStatement();
            ResultSet rs = stmt.executeQuery(sql);
            result = rs.next();
            rs.close();
            stmt.close();
        } catch (SQLException e)
        {
            e.printStackTrace();
        }
        return result;
     }
     
     /**
      * Inserts the property to the DB.
      * <BR>
      * <B>Remark<\B> : this method can not insert a property at
      * level = 0.
      * @param connection the connection to the DB.
      */
     public void insertToDB(Connection connection)
     {
        if (level == 0)
            return;
            
        int prid = getPrid(connection);
        
        tf = new TextFormator();
        String tableName = "";
        String valuesName = "";

        String values = id+", "+prid+", ";

        switch (level)
        {
            case (USER) :
                tableName = "USER_";
                valuesName += "USER_UID,";
                break;
            default :
                tableName = "PRES_";
                valuesName = "PRES_PID,";
                break;
        }

        switch (type)
        {
            case (COLOR) :
                tableName += "COLOR_PROPERTY, COLOR_PROPERTY ";
                valuesName += "COLOR_PROP_PRID, VALUE_CID";
                values += value.toString();
                break;
            case (IMAGE) :
                tableName += "IMAGE_PROPERTY, IMAGE_PROPERTY ";
                valuesName += "IMAGE_PROP_PRID, VALUE_IID";
                values += value.toString();
                break;
            default :
                tableName += "GENERAL_PROPERTY, GENERAL_PROPERTY ";
                valuesName += "GEN_PROP_PRID, VALUE";
                values += "'"+tf.prime(value)+"'";
                break;
        }
        
        String sql = "INSERT INTO "+tableName+" ("+valuesName+") ";
        sql += "VALUES("+values+")";
        
        if (DEBUG)
            System.out.println(sql);
      
        try
        {
            Statement sqlStatement = connection.createStatement();
            int i = sqlStatement.executeUpdate(sql);
        	sqlStatement.close();
        } catch (SQLException e)
        {
            e.printStackTrace();
        }
     }
     
     /**
      * Update the property in the DB.
      * @param connection the connection to the DB.
      */
     public void updateToDB(Connection connection)
     {
     }
     
     /**
      * Gets the property (if exists) from the DB.
      * <p><b>Remark</b> : if the property is not
      * set in the concerned level, this method returns <i>null</i>.
      * @param connection the connection to the DB.
      * @return null if the property is not set.
      */
     public String getFromDB(Connection connection)
     {
        String result = "";
        
        tf = new TextFormator();
        String identificatorName = "";
        String pridName = "";
        String selectValue = "";
        String tableName = "";
        String joinTableName = "";
        String sql="";
        
        switch (level)
        {
            case (DEFAULT) :
                identificatorName = "";
                selectValue = "DEF_VALUE";
                break;
            case (USER) :
                identificatorName = " AND USER_UID="+id;
                selectValue = "VALUE";
                tableName = "USER_";
                break;
            default :
                identificatorName = " AND PRES_PID="+id;
                selectValue = "VALUE";
                tableName = "PRES_";
                break;
        }

        switch (type)
        {
            case (COLOR) :
                tableName += "COLOR_PROPERTY ";
                joinTableName += ", COLOR_PROPERTY";
                selectValue += "_CID";
                pridName = "COLOR_PROP_PRID";
                break;
            case (IMAGE) :
                tableName += "IMAGE_PROPERTY ";
                joinTableName += ", IMAGE_PROPERTY";
                selectValue += "_IID";
                pridName = "IMAGE_PROP_PRID";
                break;
            default :
                tableName += "GENERAL_PROPERTY ";
                joinTableName += ", GENERAL_PROPERTY";
                pridName = "GEN_PROP_PRID";
                break;
        }
        switch (level) {
            case (DEFAULT):
                    sql = "SELECT "+selectValue+" FROM "+tableName;
                    sql += " WHERE SEARCH_NAME='"+tf.prime(name)+"'";
                    break;

            default:
                    sql = "SELECT "+selectValue+" FROM "+tableName+joinTableName;
                    sql += " WHERE SEARCH_NAME='"+tf.prime(name)+"'";
                    sql += identificatorName;
                    sql += " AND PRID="+pridName;
                    break;

        }

        if (DEBUG)
            System.out.println(sql);
      
// ExceptionHandler.debugInfo("SQL:"+sql);
       try
        {
            Statement stmt = connection.createStatement();
            ResultSet rs = stmt.executeQuery(sql);
            if (rs.next())
            {
                if (type>1)
                    result = rs.getString(1);
                else
                    result = (new Integer(rs.getInt(1))).toString();
            }
            rs.close();
            stmt.close();
        } catch (SQLException e)
        {
            ExceptionHandler.handleException(e,"getDBValue");
//            e.printStackTrace();
        }
        
        return result;
     }
     
     /**
      * Gets the property from the DB.
      * <p><b>Remark</b> : if the property is not
      * set in the concerned level, this method looks is the upper levels.
      * @param uid the identifier of the user that wants to retrieve 
      * this property.
      * @param pid the identifier of the presentation in which is the
      * foil for which the property is set.
      * @param fid the identifier of the foil for which the property
      * is set.
      * @param connection the connection to the DB.
      * @return the value of the property. <i>null</i> if
      * the property doesn't exist in the DB.
      */
     public String getValueFromDB(int uid,
                                  int pid,
                                  int fid,
                                  Connection connection)
     {
        String result = "";
        int old_level = level;
        while ((old_level !=-1) && (result.equals("")))
        {   
            switch(old_level)
            {
                case 3 :
                    dbProperty prop3 = new dbProperty(name,
                                                     value,
                                                     fid,
                                                     3,
                                                     type);
                    result = prop3.getFromDB(connection);
                    break;
                case 2 :
                    dbProperty prop2 = new dbProperty(name,
                                                     value,
                                                     pid,
                                                     2,
                                                     type);
                    result = prop2.getFromDB(connection);
                    break;
                case 1 :
                    dbProperty prop1 = new dbProperty(name,
                                                     value,
                                                     uid,
                                                     1,
                                                     type);
                    result = prop1.getFromDB(connection);
                    break;
                case 0 :
                    dbProperty prop0 = new dbProperty(name,
                                                     value,
                                                     -1,
                                                     0,
                                                     type);
                    result = prop0.getFromDB(connection);
                    break;
            }
            old_level--;
        }
        
        return result;
     }
 
     /**
      * Return the PRID corresponding to a given SEARCH_NAME and
      * a given <i>type</i>.
      * @param connection the connection to the DB.
      * @return the PRID. -1 if no property exists.
      */
     protected int getPrid(Connection connection)
     {
        int result = -1;
        String tableName = "";
        
        switch (type)
        {
            case (COLOR) :
                tableName += "COLOR_PROPERTY";
                break;
            case (IMAGE) :
                tableName += "IMAGE_PROPERTY";
                break;
            default :
                tableName += "GENERAL_PROPERTY";
                break;
        }
        
        String sql = "SELECT PRID FROM "+tableName;
        sql += " WHERE SEARCH_NAME='"+tf.prime(name)+"'";
        
        if (DEBUG)
            System.out.println(sql);
      
        try
        {
            Statement stmt = connection.createStatement();
            ResultSet rs = stmt.executeQuery(sql);
            if (rs.next())
                result = rs.getInt(1);
            rs.close();
            stmt.close();
        } catch (SQLException e)
        {
            e.printStackTrace();
        }
        return result;
     }
}